#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import pandas as pd
from carbonium import *

if __name__ == '__main__':

    name_list = [
        {
            "domains": ["raw", "output"],
            "name": "var1",
            "alias": "column_1_name",
            "output_order": 1,
            "filling_value": 10,
        },
        {
            "domains": ["raw"],
            "name": "var2",
            "alias": "column_2_name",
            "output_order": 2,
        },
        {
            "domains": ["new", "output"],
            "name": "new_var",
            "alias": "new_column_name",
        },
    ]

    structure = Structure(name_list)

    print(structure.names)
    print(structure.domains)

    print(
        structure.var1.name,
        structure.var1.domains,
        structure.var1.output_order
    )

    print(structure.get_names('raw'))

    ordered_raw_columns = [
            (
                i,
                structure.get(i).output_order,
                structure.get(i).get("filling_value")
            )
            for i in structure.get_names('raw')
    ]

    ordered_raw_columns = sorted(
        ordered_raw_columns,
        key=lambda x: x[1]
    )

    df = pd.DataFrame([
        {"var1": 100, "var2": 200},
        {"var2": 220},
    ])

    for name in structure.get_names('raw'):
        filling = structure.get(name).get("filling_value")
        if filling:
            df[name].fillna(filling, inplace=True)

    for name in structure.get_names('new'):
        df[name] = "arbitrary"

    output_columns = structure.get_names('new')
    df[output_columns].to_parquet('output.parquet')


#!/usr/bin/env python3
# -*- coding: utf-8 -*-


class Name:
    def __init__(self, domains, name, alias=None, **kwargs):
        """Name class

        Name is the building block of a carbonium structure.
        It provides an object with few mandatory arguments and
        several, arbitrary, custom properties.

        Args:
            domains (list of str): a list of string that define a multilevel name structure
            name (str): the name of the Name
            alias (str): an alternative name, that can be varied without impact on the code
                that refers to Name.name attribute
            **kwargs:
        """
        self.domains = domains
        self.name = name
        self.alias = alias
        self.properties = ["domains", "name", "alias"]

        for key in kwargs:
            self.properties.append(key)
            self.__setattr__(key, kwargs[key])

    @property
    def domains(self):
        return self._domains

    @domains.setter
    def domains(self, value):
        if not value:
            raise ValueError("domains argument cannot be None")
        if not isinstance(value, list):
            value = [value]
        self._domains = value

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, value):
        if not value:
            raise ValueError("name argument cannot be None")
        self._name = value

    @property
    def alias(self):
        return self._alias

    @alias.setter
    def alias(self, value):
        if not value:
            self._alias = self.name
        self._alias = value

    def get(self, name, default=None):
        return self.__dict__.get(name, default)

    def __str__(self):
        msg = []
        for attr in self.properties:
            msg.append("{} {}".format(attr, getattr(self, attr)))
        return "\n".join(msg)

    def __call__(self):
        return getattr(self, "alias")


from setuptools import setup

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="carbonium",
    version="0.10.2",
    url="https://github.com/pietrogiuffrida/carbonium/",
    author="Pietro Giuffrida",
    author_email="pietro.giuffri@gmail.com",
    license="MIT",
    packages=["carbonium"],
    zip_safe=False,
    install_requires=[],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    description="manage a list of names with several "
    "properties and (overlapping) order criteria",
    long_description=long_description,
    long_description_content_type="text/markdown",
)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import logging
import os
import pickle

from .name import Name

logger = logging.getLogger(__name__)
logger.setLevel("DEBUG")


class Structure:
    """Metaclass that creates a collection of :meth:`Name<crif_pyton_libraries.check_columns.Name>` instances

    For each dictionary inside columns list, Columns create an istance of
    :meth:`Name<crif_pyton_libraries.check_columns.Name>` and set its properties.
    Each Name instance is also treated as a Columns property.

    Attributes:
        names (list): an ordered list of all the names contained in columns list.
        domains (set): a set of available domains
    """

    def __init__(self, name_list):
        """Constructor method for Columns

        Args:
            name_list (list): Each dictionary inside columns list
                must respect :envvar:`ordered_columns` syntax.
        """
        self.names = []
        self.domains = set()
        for name_definition in name_list:

            if name_definition["name"] in self.names:
                raise ValueError(
                    "Name {} already present".format(name_definition["name"])
                )

            self._add_name(name_definition)

            for domain in name_definition["domains"]:
                self._add_domain(domain=domain)

    def _add_domain(self, domain):
        self.domains.add(domain)

    def _add_name(self, name_definition):
        name = Name(**name_definition)
        self.names.append(name.name)
        setattr(self, name.name, name)

    def get_names(self, domain=None):
        """Select column names from self.names respecting insertion order.

        Domain and selected arguments can be used to select a sample of names.

        Args:
            domain (str): domain of expected names. Domains refers to domain
                property of :meth:`Name<crif_pyton_libraries.check_columns.Name>`

        Returns:
            selection (list): list of selected columns.

        """

        selection = []
        for name in self.names:

            target = getattr(self, name)

            if not isinstance(target, Name):
                continue

            if domain and domain not in target.domains:
                continue

            selection.append(name)

        return selection

    def __getitem__(self, name):
        return self.__dict__[name]

    def get(self, name, default=None):
        return self.__dict__.get(name, default)

    def dump(self, filename: str, overwrite=False):
        if not filename.endswith(".pkl"):
            filename += ".pkl"

        logger.info("Saving on {}".format(filename))

        if os.path.exists(filename):
            if not overwrite:
                msg = "{} ALREADY EXISTS! I'M NOT ABLE TO OVERWRITE!".format(filename)
                logger.info(msg)
                raise FileExistsError(msg)

            logger.info("Overwriting {}...".format(filename))

        with open(filename, "wb") as output:
            pickle.dump(self.__dict__, output)

    def load(self, filename):
        logger.debug("Loading from {}".format(filename))
        if not os.path.exists(filename):
            logger.error("{} DOES NOT EXISTS!".format(filename))
            raise FileNotFoundError("{} DOES NOT EXISTS!".format(filename))

        with open(filename, "rb") as input_file:
            self.__dict__ = pickle.load(input_file)
